@@ -1,20 +1,24 @@
 /*
- * Copyright (c) 2013, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 of the
- * License.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, version 2.0,
+ * as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have included with MySQL.
+ * This program is distributed in the hope that it will be useful,  but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+ * the GNU General Public License, version 2.0, for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301  USA
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
  */
 
 #include "wb_sql_editor_form.h"
@@ -62,6 +66,8 @@ using namespace base;
 
 DEFAULT_LOG_DOMAIN("SqlResult")
 
+//----------------------------------------------------------------------------------------------------------------------
+
 class SqlEditorResult::DockingDelegate : public mforms::TabViewDockingPoint {
   mforms::TabSwitcher *_switcher;
 
@@ -86,16 +92,17 @@ public:
   }
 };
 
+//----------------------------------------------------------------------------------------------------------------------
+
 SqlEditorResult::SqlEditorResult(SqlEditorPanel *owner)
-  : mforms::AppView(true, "QueryResult", false),
+  : mforms::AppView(true, "Query Result", "QueryResult", false),
     _owner(owner),
     _tabview(mforms::TabViewTabless),
     _switcher(mforms::VerticalIconSwitcher),
     _tabdock_delegate(new DockingDelegate(&_tabview, &_switcher, std::string("SqlResultPanel"))),
     _tabdock(_tabdock_delegate, true),
-    _pinned(false)
+    _pinned(false) {
 
-{
   _result_grid = NULL;
   _grid_header_menu = NULL;
   _column_info_menu = NULL;
@@ -103,11 +110,18 @@ SqlEditorResult::SqlEditorResult(SqlEditorPanel *owner)
   _query_stats_created = false;
   _form_view_created = false;
 
+  _column_info_box = nullptr;
+  _query_stats_box = nullptr;
+  _execution_plan_placeholder = nullptr;
+  _query_stats_panel = nullptr;
+  _form_result_view = nullptr;
+
   _spatial_view_initialized = false;
   _spatial_result_view = NULL;
 
   add(&_tabview, true, true);
 
+  _switcher.set_name("Resultset View Switcher");
   _switcher.attach_to_tabview(&_tabview);
   _switcher.set_collapsed(bec::GRTManager::get()->get_app_option_int("Recordset:SwitcherCollapsed", 0) != 0);
 
@@ -115,14 +129,12 @@ SqlEditorResult::SqlEditorResult(SqlEditorPanel *owner)
   _switcher.signal_changed()->connect(std::bind(&SqlEditorResult::switch_tab, this));
   _switcher.signal_collapse_changed()->connect(std::bind(&SqlEditorResult::switcher_collapsed, this));
 
-  _execution_plan_placeholder = NULL;
-
   // put a placeholder for the resultset, which will be replaced when a resultset is actually available
-  _resultset_placeholder = mforms::manage(new mforms::AppView(false, "ResultGridPlaceholder", false));
-  _resultset_placeholder->set_back_color("#ffffff");
+  _resultset_placeholder = mforms::manage(new mforms::AppView(false, "Result Grid Placeholder", "ResultGridPlaceholder", false));
   _resultset_placeholder->set_title("Result\nGrid");
   _resultset_placeholder->set_identifier("result_grid");
   _tabdock.dock_view(_resultset_placeholder, "output_type-resultset.png");
+  _tabdock.set_name("Resultset Views");
 
   {
     db_query_QueryEditorRef editor(owner->grtobj());
@@ -131,8 +143,39 @@ SqlEditorResult::SqlEditorResult(SqlEditorPanel *owner)
   }
 
   set_on_close(std::bind(&SqlEditorResult::can_close, this));
+
+  NotificationCenter::get()->add_observer(this, "GNColorsChanged");
+  updateColors();
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
+void SqlEditorResult::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info) {
+  if (name == "GNColorsChanged") {
+    updateColors();
+  }
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+void SqlEditorResult::updateColors() {
+  std::string background = base::Color::getSystemColor(TextBackgroundColor).to_html();
+  if (_resultset_placeholder != nullptr)
+    _resultset_placeholder->set_back_color(background);
+  if (_column_info_box != nullptr)
+    _column_info_box->set_back_color(background);
+  if (_query_stats_box != nullptr)
+    _query_stats_box->set_back_color(background);
+  if (_execution_plan_placeholder != nullptr)
+    _execution_plan_placeholder->set_back_color(background);
+  if (_query_stats_panel != nullptr)
+    _query_stats_panel->set_back_color(background);
+  if (_form_result_view != nullptr)
+    _form_result_view->updateColors();
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::reset_sorting() {
   Recordset::Ref rset(recordset());
   if (rset)
@@ -143,6 +186,8 @@ void SqlEditorResult::reset_sorting() {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::copy_column_name() {
   int column = _result_grid->get_clicked_header_column();
   Recordset::Ref rset(recordset());
@@ -150,6 +195,8 @@ void SqlEditorResult::copy_column_name() {
     mforms::Utilities::set_clipboard_text(rset->get_column_caption(column));
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::copy_all_column_names() {
   Recordset::Ref rset(recordset());
   if (rset) {
@@ -165,6 +212,8 @@ void SqlEditorResult::copy_all_column_names() {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::open_field_editor(int row, int column) {
   Recordset::Ref rset(recordset());
   if (rset) {
@@ -175,6 +224,8 @@ void SqlEditorResult::open_field_editor(int row, int column) {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::update_selection_for_menu_extra(mforms::ContextMenu *menu, const std::vector<int> &rows,
                                                       int column) {
   mforms::MenuItem *item = menu->find_item("edit_cell");
@@ -184,6 +235,8 @@ void SqlEditorResult::update_selection_for_menu_extra(mforms::ContextMenu *menu,
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::set_recordset(Recordset::Ref rset) {
   if (_resultset_placeholder) {
     _tabdock_delegate->undock_view(_resultset_placeholder);
@@ -217,12 +270,12 @@ void SqlEditorResult::set_recordset(Recordset::Ref rset) {
     ->connect(std::bind(&SqlEditorResult::reset_sorting, this));
 
   _grid_header_menu = new mforms::ContextMenu();
-  _grid_header_menu->add_item_with_title("Copy Field Name", std::bind(&SqlEditorResult::copy_column_name, this));
+  _grid_header_menu->add_item_with_title("Copy Field Name", std::bind(&SqlEditorResult::copy_column_name, this), "Copy Field Name", "");
   _grid_header_menu->add_item_with_title("Copy All Field Names",
-                                         std::bind(&SqlEditorResult::copy_all_column_names, this));
+                                         std::bind(&SqlEditorResult::copy_all_column_names, this), "Copy All Field Names", "");
   _grid_header_menu->add_separator();
-  _grid_header_menu->add_item_with_title("Reset Sorting", std::bind(&SqlEditorResult::reset_sorting, this));
-  _grid_header_menu->add_item_with_title("Reset Column Widths", std::bind(&SqlEditorResult::reset_column_widths, this));
+  _grid_header_menu->add_item_with_title("Reset Sorting", std::bind(&SqlEditorResult::reset_sorting, this), "Reset Sorting", "");
+  _grid_header_menu->add_item_with_title("Reset Column Widths", std::bind(&SqlEditorResult::reset_column_widths, this), "Reset Column Widths", "");
 
   
   std::string fontDescription = bec::GRTManager::get()->get_app_option_string("workbench.general.Resultset:Font");
@@ -257,15 +310,23 @@ void SqlEditorResult::set_recordset(Recordset::Ref rset) {
   rset->data_edited_signal.connect(std::bind(&mforms::View::set_needs_repaint, grid));
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 SqlEditorResult::~SqlEditorResult() {
+  base::NotificationCenter::get()->remove_observer(this);
+
   delete _column_info_menu;
   delete _grid_header_menu;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 Recordset::Ref SqlEditorResult::recordset() const {
   return _rset.lock();
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 bool SqlEditorResult::has_pending_changes() {
   Recordset::Ref rset(recordset());
   if (rset)
@@ -273,24 +334,32 @@ bool SqlEditorResult::has_pending_changes() {
   return false;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::apply_changes() {
   Recordset::Ref rset(recordset());
   if (rset)
     rset->apply_changes();
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::discard_changes() {
   Recordset::Ref rset(recordset());
   if (rset)
     rset->rollback();
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 std::string SqlEditorResult::caption() const {
   RETVAL_IF_FAIL_TO_RETAIN_WEAK_PTR(Recordset, _rset, rs, "") {
     return rs->caption();
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 std::vector<SpatialDataView::SpatialDataSource> SqlEditorResult::get_spatial_columns() {
   std::vector<SpatialDataView::SpatialDataSource> spatial_columns;
   int i = 0;
@@ -311,11 +380,15 @@ std::vector<SpatialDataView::SpatialDataSource> SqlEditorResult::get_spatial_col
   return spatial_columns;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::set_title(const std::string &title) {
   grtobj()->name(title);
   mforms::AppView::set_title(title);
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 bool SqlEditorResult::can_close() {
   if (Recordset::Ref rs = recordset())
     if (!rs->can_close(true))
@@ -327,6 +400,8 @@ bool SqlEditorResult::can_close() {
   return true;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::close() { // called by DockingPoint::close_view()
   if (Recordset::Ref rs = recordset())
     rs->close();
@@ -335,6 +410,8 @@ void SqlEditorResult::close() { // called by DockingPoint::close_view()
   mforms::AppView::close();
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::switch_tab() {
   mforms::AppView *tab = _tabdock.selected_view();
 
@@ -354,7 +431,8 @@ void SqlEditorResult::switch_tab() {
     } else if (tab->identifier() == "result_grid") {
       if (_resultset_placeholder) {
         _owner->owner()->exec_editor_sql(_owner, true, true, true, false, this);
-        set_title(_rset.lock()->caption());
+        if (!_rset.expired())
+          set_title(_rset.lock()->caption());
       }
     } else if (tab->identifier() == "execution_plan") {
       if (_execution_plan_placeholder) {
@@ -383,14 +461,19 @@ void SqlEditorResult::switch_tab() {
       _spatial_result_view->activate();
     }
   }
+
+  updateColors();
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::add_switch_toggle_toolbar_item(mforms::ToolBar *tbar) {
   _collapse_toggled_sig.disconnect();
   mforms::App *app = mforms::App::get();
   tbar->add_item(mforms::manage(new mforms::ToolBarItem(mforms::ExpanderItem)));
   mforms::ToolBarItem *item = mforms::manage(new mforms::ToolBarItem(mforms::ToggleItem));
-  item->set_name("sidetoggle");
+  item->set_name("Side Toggle");
+  item->setInternalName("sidetoggle");
   item->set_icon(app->get_resource_path("output_type-toggle-on.png"));
   item->set_alt_icon(app->get_resource_path("output_type-toggle-off.png"));
   item->signal_activated()->connect(std::bind(&SqlEditorResult::toggle_switcher_collapsed, this));
@@ -400,6 +483,8 @@ void SqlEditorResult::add_switch_toggle_toolbar_item(mforms::ToolBar *tbar) {
     _collapse_toggled.connect(std::bind(&mforms::ToolBarItem::set_checked, item, std::placeholders::_1));
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::switcher_collapsed() {
   bool state = _switcher.get_collapsed();
   for (std::list<mforms::ToolBar *>::const_iterator it = _toolbars.begin(); it != _toolbars.end(); ++it) {
@@ -410,6 +495,8 @@ void SqlEditorResult::switcher_collapsed() {
   bec::GRTManager::get()->set_app_option("Recordset:SwitcherCollapsed", grt::IntegerRef(state ? 1 : 0));
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::show_export_recordset() {
   try {
     RETURN_IF_FAIL_TO_RETAIN_WEAK_PTR(Recordset, _rset, rs) {
@@ -439,30 +526,35 @@ void SqlEditorResult::show_export_recordset() {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::show_import_recordset() {
   try {
     RETURN_IF_FAIL_TO_RETAIN_WEAK_PTR(Recordset, _rset, rs) {
       grt::BaseListRef args(true);
-
-      if (result_grtobj().is_valid()) {
-        args.ginsert(result_grtobj());
-        grt::Module *module = grt::GRT::get()->get_module("SQLIDEUtils");
-        if (module)
-          module->call_function("importRecordsetDataFromFile", args);
-      } else
-        logFatal("resultset GRT obj is NULL\n");
+      grt::Module *mod = grt::GRT::get()->get_module("SQLIDEUtils");
+      if (mod) {
+        args.ginsert(_owner->owner()->grtobj());
+        mod->call_function("launchPowerImport", args);
+      }
+      else
+        logFatal("Unable to launch import wizard\n");
     }
   } catch (const std::exception &exc) {
     mforms::Utilities::show_error("Error importing recordset", exc.what(), "OK");
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::toggle_switcher_collapsed() {
   bool flag = !_switcher.get_collapsed();
   _switcher.set_collapsed(flag);
   _collapse_toggled(flag);
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::on_recordset_column_resized(int column) {
   if (column >= 0) {
     std::string column_id = _column_width_storage_ids[column];
@@ -471,6 +563,8 @@ void SqlEditorResult::on_recordset_column_resized(int column) {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::onRecordsetColumnsResized(const std::vector<int> cols) {
   std::vector<int>::const_iterator it;
   std::map<std::string, int> widths;
@@ -489,6 +583,8 @@ void SqlEditorResult::onRecordsetColumnsResized(const std::vector<int> cols) {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::reset_column_widths() {
   ColumnWidthCache *cache = _owner->owner()->column_width_cache();
 
@@ -508,6 +604,8 @@ void SqlEditorResult::reset_column_widths() {
   restore_grid_column_widths();
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 std::vector<float> SqlEditorResult::get_autofit_column_widths(Recordset *rs) {
   std::vector<float> widths(rs->get_column_count());
   std::string font = bec::GRTManager::get()->get_app_option_string("workbench.general.Resultset:Font");
@@ -527,6 +625,8 @@ std::vector<float> SqlEditorResult::get_autofit_column_widths(Recordset *rs) {
   return widths;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::restore_grid_column_widths() {
   ColumnWidthCache *cache = _owner->owner()->column_width_cache();
 
@@ -561,16 +661,21 @@ void SqlEditorResult::restore_grid_column_widths() {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::dock_result_grid(mforms::GridView *view) {
   _result_grid = view;
-  view->set_name("result-grid-wrapper");
+  view->set_name("Result Grid Wrapper");
+  view->setInternalName("result-grid-wrapper");
 
   mforms::AppView *grid_view;
   {
-    mforms::AppView *box = grid_view = mforms::manage(new mforms::AppView(false, "ResultGridView", false));
-    box->set_name("resultset-host");
+    mforms::AppView *box = grid_view = mforms::manage(new mforms::AppView(false, "Result Grid View", "ResultGridView", false));
+    box->set_name("Resultset Host");
+    box->setInternalName("resultset-host");
     mforms::ToolBar *tbar = _rset.lock()->get_toolbar();
-    tbar->set_name("resultset-toolbar");
+    tbar->set_name("Resultset Toolbar");
+    tbar->setInternalName("resultset-toolbar");
     _toolbars.push_back(tbar);
 
     add_switch_toggle_toolbar_item(tbar);
@@ -593,15 +698,13 @@ void SqlEditorResult::dock_result_grid(mforms::GridView *view) {
     _tabdock.dock_view(_form_result_view, "output_type-formeditor.png");
   }
   {
-    _column_info_box = mforms::manage(new mforms::AppView(false, "ResultFieldTypes", false));
-    _column_info_box->set_back_color("#ffffff");
+    _column_info_box = mforms::manage(new mforms::AppView(false, "Result Field Types", "ResultFieldTypes", false));
     _column_info_box->set_title("Field\nTypes");
     _column_info_box->set_identifier("column_info");
     _tabdock.dock_view(_column_info_box, "output_type-fieldtypes.png");
   }
   {
-    _query_stats_box = mforms::manage(new mforms::AppView(false, "ResultQueryStats", false));
-    _query_stats_box->set_back_color("#ffffff");
+    _query_stats_box = mforms::manage(new mforms::AppView(false, "Result Query Stats", "ResultQueryStats", false));
     _query_stats_box->set_title("Query\nStats");
     _query_stats_box->set_identifier("query_stats");
     _tabdock.dock_view(_query_stats_box, "output_type-querystats.png");
@@ -626,8 +729,7 @@ void SqlEditorResult::dock_result_grid(mforms::GridView *view) {
     }
   }
   if (!has_explain_tab) {
-    _execution_plan_placeholder = mforms::manage(new mforms::AppView(false, "ExecutionPlan", false));
-    _execution_plan_placeholder->set_back_color("#ffffff");
+    _execution_plan_placeholder = mforms::manage(new mforms::AppView(false, "Execution Plan", "ExecutionPlan", false));
     _execution_plan_placeholder->set_title("Execution\nPlan");
     _execution_plan_placeholder->set_identifier("execution_plan");
     _tabdock.dock_view(_execution_plan_placeholder, "output_type-executionplan.png");
@@ -635,6 +737,8 @@ void SqlEditorResult::dock_result_grid(mforms::GridView *view) {
   _switcher.set_selected(0);
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::create_spatial_view_panel_if_needed() {
   if (Recordset::Ref rset = _rset.lock()) {
     Recordset_cdbc_storage::Ref storage(std::dynamic_pointer_cast<Recordset_cdbc_storage>(rset->data_storage()));
@@ -658,16 +762,19 @@ void SqlEditorResult::create_spatial_view_panel_if_needed() {
       }
       _spatial_result_view = mforms::manage(new SpatialDataView(this));
       add_switch_toggle_toolbar_item(_spatial_result_view->get_toolbar());
-      mforms::AppView *box = mforms::manage(new mforms::AppView(false, "SpatialView", false));
+      mforms::AppView *box = mforms::manage(new mforms::AppView(false, "Spatial View", "SpatialView", false));
       box->set_title("Spatial\nView");
       box->set_identifier("spatial_result_view");
-      box->set_name("spatial-host");
+      box->set_name("Spatial Host");
+      box->setInternalName("spatial-host");
       box->add(_spatial_result_view, true, true);
       _tabdock.dock_view(box, "output_type-spacialview.png");
     }
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static std::string format_ps_time(std::int64_t t) {
   int hours, mins;
   double secs;
@@ -679,12 +786,16 @@ static std::string format_ps_time(std::int64_t t) {
   return base::strfmt("%i:%02i:%02.8f", hours, mins, secs);
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static mforms::Label *bold_label(const std::string text) {
   mforms::Label *l = mforms::manage(new mforms::Label(text));
   l->set_style(mforms::BoldStyle);
   return l;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::copy_column_info(mforms::TreeView *tree) {
   std::list<mforms::TreeNodeRef> nodes(tree->get_selection());
   std::string text;
@@ -702,6 +813,8 @@ void SqlEditorResult::copy_column_info(mforms::TreeView *tree) {
   mforms::Utilities::set_clipboard_text(text);
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::copy_column_info_name(mforms::TreeView *tree) {
   std::list<mforms::TreeNodeRef> nodes(tree->get_selection());
   std::string text;
@@ -712,6 +825,8 @@ void SqlEditorResult::copy_column_info_name(mforms::TreeView *tree) {
   mforms::Utilities::set_clipboard_text(text);
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::create_column_info_panel() {
   RETURN_IF_FAIL_TO_RETAIN_WEAK_PTR(Recordset, _rset, rs) {
     Recordset_cdbc_storage::Ref storage(std::dynamic_pointer_cast<Recordset_cdbc_storage>(rs->data_storage()));
@@ -746,9 +861,9 @@ void SqlEditorResult::create_column_info_panel() {
       tree->set_selection_mode(mforms::TreeSelectMultiple);
 
       _column_info_menu = new mforms::ContextMenu();
-      _column_info_menu->add_item_with_title("Copy", std::bind(&SqlEditorResult::copy_column_info, this, tree));
+      _column_info_menu->add_item_with_title("Copy", std::bind(&SqlEditorResult::copy_column_info, this, tree), "Copy", "");
       _column_info_menu->add_item_with_title("Copy Name",
-                                             std::bind(&SqlEditorResult::copy_column_info_name, this, tree));
+                                             std::bind(&SqlEditorResult::copy_column_info_name, this, tree), "Copy Name", "");
       tree->set_context_menu(_column_info_menu);
 
       int i = 0;
@@ -771,6 +886,8 @@ void SqlEditorResult::create_column_info_panel() {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static struct ColorDefinitions {
   double r, g, b;
 } colors[] = {
@@ -848,6 +965,8 @@ static std::string render_stages(std::vector<SqlEditorForm::PSStage> &stages) {
   return path;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static std::string render_waits(std::vector<SqlEditorForm::PSWait> &waits) {
   std::string path = mforms::Utilities::get_special_folder(mforms::ApplicationData) + "/waits.png";
 
@@ -909,18 +1028,19 @@ static std::string render_waits(std::vector<SqlEditorForm::PSWait> &waits) {
   return path;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::create_query_stats_panel() {
   RETURN_IF_FAIL_TO_RETAIN_WEAK_PTR(Recordset, _rset, rs) {
     SqlEditorForm::RecordsetData *rsdata = dynamic_cast<SqlEditorForm::RecordsetData *>(rs->client_data());
     std::string info;
 
-    mforms::ScrollPanel *spanel = mforms::manage(new mforms::ScrollPanel());
+    _query_stats_panel = mforms::manage(new mforms::ScrollPanel());
     mforms::Table *table = mforms::manage(new mforms::Table());
     table->set_padding(20);
     table->set_row_count(6);
     table->set_column_count(2);
     table->set_row_spacing(4);
-    spanel->set_back_color("#ffffff");
 
     mforms::ToolBar *tbar = mforms::manage(new mforms::ToolBar(mforms::SecondaryToolBar));
     _toolbars.push_back(tbar);
@@ -972,8 +1092,8 @@ void SqlEditorResult::create_query_stats_panel() {
 
       box->add(bold_label("Rows Processed:"), false, true);
       info.clear();
-      info.append(strfmt("Rows affected: %lli\n", ps_stats["ROWS_AFFECTED"])),
-        info.append(strfmt("Rows sent to client: %lli\n", ps_stats["ROWS_SENT"]));
+      info.append(strfmt("Rows affected: %lli\n", ps_stats["ROWS_AFFECTED"]));
+      info.append(strfmt("Rows sent to client: %lli\n", ps_stats["ROWS_SENT"]));
       info.append(strfmt("Rows examined: %lli\n", ps_stats["ROWS_EXAMINED"]));
       box->add(mforms::manage(new mforms::Label(info)), false, true);
 
@@ -1052,12 +1172,14 @@ void SqlEditorResult::create_query_stats_panel() {
       table->add(image, 0, 2, 5, 6, mforms::HFillFlag | mforms::VFillFlag);
     }
 
-    spanel->add(table);
+    _query_stats_panel->add(table);
 
-    _query_stats_box->add(spanel, true, true);
+    _query_stats_box->add(_query_stats_panel, true, true);
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void SqlEditorResult::view_record_in_form(int row_id) {
   if (_form_result_view) {
     _tabview.set_active_tab(1);
@@ -1065,3 +1187,5 @@ void SqlEditorResult::view_record_in_form(int row_id) {
     _form_result_view->display_record(row_id);
   }
 }
+
+//----------------------------------------------------------------------------------------------------------------------
