@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -155,8 +155,8 @@ Connection::Connection(mdc::Layer *layer, FigureEventHub *hub, model_Object *rep
   _start_figure = 0;
   _end_figure = 0;
 
-  set_pen_color(base::Color::getSystemColor(base::TextColor));
-  set_fill_color(base::Color::getSystemColor(base::TextBackgroundColor));
+  set_pen_color(base::Color::black());
+  set_fill_color(base::Color::white());
 }
 
 void Connection::set_start_figure(mdc::CanvasItem *item) {
@@ -340,6 +340,7 @@ void Connection::stroke_outline(mdc::CairoCtx *cr, float offset) const {
 //--------------------------------------------------------------------------------------------------
 
 void Connection::stroke_outline_gl(float offset) const {
+#ifndef __APPLE__
   if (!_split || _segments.size() < 2)
     mdc::Line::stroke_outline_gl(offset);
   else {
@@ -387,6 +388,7 @@ void Connection::stroke_outline_gl(float offset) const {
 
     glEnd();
   }
+#endif
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -665,6 +667,7 @@ void Connection::render(mdc::CairoCtx *cr) {
 //--------------------------------------------------------------------------------------------------
 
 void Connection::render_gl(mdc::CairoCtx *cr) {
+#ifndef __APPLE__
   if (_segments.empty())
     return;
 
@@ -755,6 +758,7 @@ void Connection::render_gl(mdc::CairoCtx *cr) {
   }
 
   glPopMatrix();
+#endif
 }
 
 //--------------------------------------------------------------------------------------------------
