@@ -1,20 +1,24 @@
 /*
- * Copyright (c) 2010, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 of the
- * License.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, version 2.0,
+ * as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have included with MySQL.
+ * This program is distributed in the hope that it will be useful,  but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+ * the GNU General Public License, version 2.0, for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301  USA
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
  */
 
 #ifndef __GRT_SHELL_WINDOW_H__
@@ -98,7 +102,7 @@ protected:
   mforms::Box _content;
   mforms::Box _padding_box;
   mforms::Splitter _hsplitter;
-#ifdef _WIN32
+#ifdef _MSC_VER
   mforms::Panel _side_header_panel;
 #endif
   mforms::TabView _side_tab;
@@ -142,7 +146,7 @@ protected:
   mforms::TextEntry _shell_entry;
 
   mforms::TabView _lower_tab;
-#ifdef _WIN32
+#ifdef _MSC_VER
   mforms::Panel _lower_header_panel;
 #endif
   mforms::TextBox _output_text;
@@ -162,6 +166,9 @@ protected:
   PythonDebugger *_debugger;
 
   std::vector<GRTCodeEditor *> _editors;
+  
+  bool _userSnippetsLoaded;
+  bool _snippetClicked;
 
   void refresh_all();
   void side_tab_changed();
@@ -258,3 +265,4 @@ private:
 };
 
 #endif
+
