@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -469,8 +469,8 @@ static void addTextToOutput(const std::string &text, bool bring_to_front, void *
         break;
       }
       case 3:
-        [[NSPasteboard generalPasteboard] declareTypes: @[NSStringPboardType] owner:nil];
-        [[NSPasteboard generalPasteboard] setString: [NSString stringWithCPPString: sql] forType: NSStringPboardType];
+        [[NSPasteboard generalPasteboard] declareTypes: @[NSPasteboardTypeString] owner:nil];
+        [[NSPasteboard generalPasteboard] setString: [NSString stringWithCPPString: sql] forType: NSPasteboardTypeString];
         break;
     }
   }
