@@ -1,20 +1,24 @@
 /*
- * Copyright (c) 2007, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 of the
- * License.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, version 2.0,
+ * as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have included with MySQL.
+ * This program is distributed in the hope that it will be useful,  but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+ * the GNU General Public License, version 2.0, for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301  USA
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "grtpp_shell_python.h"
@@ -37,6 +41,7 @@
 
 #include "serializer.h"
 #include "unserializer.h"
+#include <iostream>
 
 DEFAULT_LOG_DOMAIN(DOMAIN_GRT)
 
@@ -203,7 +208,31 @@ type_error::type_error(const std::string &expected, const std::string &actual, T
 db_error::db_error(const sql::SQLException &exc) : std::runtime_error(exc.what()), _error(exc.getErrorCode()) {
 }
 
-//--------------------------------------------------------------------------------------------------
+//----------------- Value ----------------------------------------------------------------------------------------------
+
+internal::Value* internal::Value::retain() {
+  g_atomic_int_inc(&_refcount);
+  return this;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+void internal::Value::release() {
+#ifdef WB_DEBUG
+  if (_refcount == 0)
+    logWarning("GRT: releasing invalid object\n");
+#endif
+  if (g_atomic_int_dec_and_test(&_refcount))
+    delete this;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+base::refcount_t internal::Value::refcount() const {
+  return g_atomic_int_get(&_refcount);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
 
 StringRef StringRef::format(const char *format, ...) {
   va_list args;
@@ -222,7 +251,7 @@ StringRef StringRef::format(const char *format, ...) {
 
 //--------------------------------------------------------------------------------------------------
 
-GRT::GRT() : _check_serialized_crc(false), _verbose(false) {
+GRT::GRT() : _check_serialized_crc(false), _verbose(false), _testing(false) {
   _scanning_modules = false;
 
   _tracking_changes = 0;
@@ -244,8 +273,17 @@ GRT::GRT() : _check_serialized_crc(false), _verbose(false) {
 }
 
 GRT::~GRT() {
+
+  for (auto &it: _messageSlotStack) {
+    delete it;
+  }
+   _messageSlotStack.clear();
+
   delete _shell;
+  _shell = nullptr;
+
   delete _default_undo_manager;
+  _default_undo_manager = nullptr;
 
   // We need to first release PythonLoader so we don't end up Python calling some WB modules
   for (std::list<ModuleLoader *>::iterator iter = _loaders.begin(); iter != _loaders.end(); ++iter) {
@@ -256,11 +294,12 @@ GRT::~GRT() {
     }
   }
 
-  for (std::vector<Module *>::iterator iter = _modules.begin(); iter != _modules.end(); ++iter) {
-    auto mod = (*iter)->getGModule();
-    delete *iter;
-    if (mod)
-      g_module_close(mod);
+  for (const auto &it: _modules) {
+    auto module = it->getModule();
+    delete it;
+    if (module) {
+      g_module_close(module);
+    }
   }
 
   _modules.clear();
@@ -314,7 +353,7 @@ UndoGroup *GRT::begin_undoable_action(UndoGroup *group) {
 void GRT::end_undoable_action(const std::string &group_description) {
   if (!get_undo_manager()->end_undo_group(group_description, true)) {
     if (getenv("DEBUG_UNDO"))
-      g_warning("'%s' was empty", group_description.c_str());
+      logWarning("'%s' was empty\n", group_description.c_str());
   }
   stop_tracking_changes();
 }
@@ -332,6 +371,10 @@ void GRT::unlock() const {
 
 //--------------------------------------------------------------------------------------------------
 
+bool GRT::metaclassesNeedRegister() {
+  return !internal::ClassRegistry::get_instance()->isEmpty();
+}
+
 void GRT::load_metaclasses(const std::string &file, std::list<std::string> *requires) {
   xmlNodePtr root;
   xmlDocPtr doc;
@@ -371,6 +414,74 @@ void GRT::load_metaclasses(const std::string &file, std::list<std::string> *requ
   xmlFreeDoc(doc);
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void GRT::reinitialiseForTests() {
+  delete _shell;
+  _shell = nullptr;
+  delete _default_undo_manager;
+  _default_undo_manager = nullptr;
+
+  // We need to first release PythonLoader so we don't end up Python calling some WB modules
+  for (std::list<ModuleLoader *>::iterator iter = _loaders.begin(); iter != _loaders.end(); ++iter) {
+    if ((*iter)->get_loader_name() == grt::LanguagePython) {
+      delete *iter;
+      _loaders.erase(iter);
+      break;
+    }
+  }
+
+  for (const auto &it: _modules) {
+    auto module = it->getModule();
+    delete it;
+    if (module) {
+      g_module_close(module);
+    }
+  }
+
+  _modules.clear();
+  _objects_cache.clear();
+  _cached_module_wrapper.clear();
+
+  for (std::map<std::string, Interface *>::iterator iter = _interfaces.begin(); iter != _interfaces.end(); ++iter)
+    delete iter->second;
+  _interfaces.clear();
+
+  for (std::list<ModuleLoader *>::iterator iter = _loaders.begin(); iter != _loaders.end(); ++iter)
+    delete *iter;
+  _loaders.clear();
+
+  for (std::map<std::string, MetaClass *>::iterator iter = _metaclasses.begin(); iter != _metaclasses.end(); ++iter) {
+    logDebug3("Deleting metaclass: %s\n", iter->first.c_str());
+    delete iter->second;
+  }
+  _metaclasses.clear();
+  _metaclasses_list.clear();
+
+  internal::ClassRegistry::get_instance()->cleanUp();
+  _root.clear();
+
+  _scanning_modules = false;
+
+  _tracking_changes = 0;
+
+  if (getenv("GRT_VERBOSE"))
+  _verbose = true;
+
+  GRTNotificationCenter::setup();
+
+  _default_undo_manager = new UndoManager;
+
+  add_module_loader(new CPPModuleLoader());
+
+  // register metaclass for base class
+  add_metaclass(MetaClass::create_base_class());
+
+  _root = grt::DictRef(true);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 int GRT::scan_metaclasses_in(const std::string &directory, std::multimap<std::string, std::string> *requires) {
   GDir *dir;
   const char *entry;
@@ -403,7 +514,6 @@ int GRT::scan_metaclasses_in(const std::string &directory, std::multimap<std::st
   }
 
   g_dir_close(dir);
-
   return (int)(_metaclasses.size() - old_count);
 }
 
@@ -467,8 +577,7 @@ void GRT::end_loading_metaclasses(bool check_class_binding) {
     // check if there are any metaclasses with unbound members
     for (std::map<std::string, MetaClass *>::iterator iter = _metaclasses.begin(); iter != _metaclasses.end(); ++iter) {
       if (!iter->second->is_bound())
-        g_warning(
-          "Allocation function of '%s' is unbound, which probably means the implementing C++ class was not "
+        logWarning("Allocation function of '%s' is unbound, which probably means the implementing C++ class was not"
           "registered\n",
           iter->second->name().c_str());
     }
@@ -625,7 +734,10 @@ void GRT::add_module_loader(ModuleLoader *loader) {
 }
 
 bool GRT::load_module(const std::string &path, const std::string &basePath, bool refresh) {
-  std::string shortendPath = "<base dir>/" + base::relativePath(basePath, path);
+  std::string shortendPath = base::relativePath(basePath, path);
+  if (shortendPath != path)
+    shortendPath = "<base dir>/" + shortendPath;
+
   for (std::list<ModuleLoader *>::iterator loader = _loaders.begin(); loader != _loaders.end(); ++loader) {
     if ((*loader)->check_file_extension(path)) {
       logDebug2("Trying to load module '%s' (%s)\n", shortendPath.c_str(), (*loader)->get_loader_name().c_str());
@@ -892,39 +1004,50 @@ Shell *GRT::get_shell() {
 
 //--------------------------------------------------------------------------------
 
-void GRT::push_message_handler(const MessageSlot &slot) {
+void GRT::pushMessageHandler(SlotHolder *slot) {
   base::RecMutexLock lock(_message_mutex);
-  _message_slot_stack.push_back(slot);
+  _messageSlotStack.push_back(slot);
 }
 
-void GRT::pop_message_handler() {
+void GRT::popMessageHandler() {
   base::RecMutexLock lock(_message_mutex);
-  if (_message_slot_stack.empty())
-    logError("pop_message_handler() called on empty handler stack");
-  else
-    _message_slot_stack.pop_back();
+  if (_messageSlotStack.empty()) {
+    logError("popMessageHandler() called on empty handler stack");
+  } else {
+    delete _messageSlotStack.back();
+    _messageSlotStack.pop_back();
+  }
+}
+
+void GRT::removeMessageHandler(SlotHolder *slot) {
+  base::RecMutexLock lock(_message_mutex);
+  auto iter = std::find(_messageSlotStack.begin(), _messageSlotStack.end(), slot);
+  if (iter != _messageSlotStack.end()) {
+    delete *iter;
+    _messageSlotStack.erase(iter);
+  }
 }
 
 bool GRT::handle_message(const Message &msg, void *sender) {
   // Don't log any message if there's no message slot is occupied. It just means
   // we don't want anything logged.
-  if (!_message_slot_stack.empty()) {
+  if (!_messageSlotStack.empty()) {
     int i = 0;
-    MessageSlot slot;
+    SlotHolder *slot = nullptr;
     for (;;) {
       {
         base::RecMutexLock lock(_message_mutex);
-        if ((int)_message_slot_stack.size() - i - 1 >= 0) {
-          slot = _message_slot_stack[_message_slot_stack.size() - i - 1];
+        if ((int)_messageSlotStack.size() - i - 1 >= 0) {
+          slot = _messageSlotStack[_messageSlotStack.size() - i - 1];
           ++i;
         } else
           break;
       }
-      if (slot(msg, sender))
+      if (slot->slot(msg, sender))
         return true;
     }
   }
-  logError("Unhandled message (%lu): %s\n", (unsigned long)_message_slot_stack.size(), msg.format().c_str());
+  logError("Unhandled message (%lu): %s\n", (unsigned long)_messageSlotStack.size(), msg.format().c_str());
   return false;
 }
 
@@ -942,17 +1065,6 @@ bool GRT::query_status() {
   return _status_query_slot_stack.back()();
 }
 
-void GRT::make_output_visible(void *sender) {
-  base::RecMutexLock lock(_message_mutex);
-  Message msg;
-  msg.type = ControlMsg;
-  msg.text = "show";
-  msg.detail = "";
-  msg.timestamp = time(NULL);
-  msg.progress = 0.0;
-  handle_message(msg, sender);
-}
-
 // XXX: these handlers should go and be replaced by pure log_* calls.
 void GRT::send_error(const std::string &message, const std::string &details, void *sender) {
   base::RecMutexLock lock(_message_mutex);
