@@ -1,20 +1,24 @@
 /*
- * Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 of the
- * License.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, version 2.0,
+ * as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have included with MySQL.
+ * This program is distributed in the hope that it will be useful,  but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+ * the GNU General Public License, version 2.0, for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301  USA
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "python_context.h"
@@ -39,7 +43,7 @@
 
 #include "base/log.h"
 
-#ifdef _WIN32
+#ifdef _MSC_VER
 #include "base/event_log.h"
 #endif
 
@@ -68,7 +72,7 @@ PythonContextHelper::PythonContextHelper(const std::string &module_path) {
   if (getenv("PYTHON_DEBUG"))
     Py_VerboseFlag = 5;
 
-#ifdef _WIN32
+#ifdef _MSC_VER
   // Hack needed in Windows because Python lib uses C:\Python26 as default pythonhome
   // That will cause conflicts if there is some other Python installed in there (bug #52949)
 
@@ -342,12 +346,13 @@ static PyObject *grt_print(PyObject *self, PyObject *args) {
   } else if (!ctx->pystring_to_string(o, text, true))
     return NULL;
 
-#ifdef _WIN32
+#ifdef _MSC_VER
   OutputDebugStringA(text.c_str());
 #else
   g_print("%s", text.c_str()); // g_print is not routed to g_log
 #endif
   grt::GRT::get()->send_output(text);
+  logDebug3("%s\n", text.c_str());
 
   Py_INCREF(Py_None);
   return Py_None;
@@ -410,9 +415,11 @@ static PyObject *grt_send_output(PyObject *self, PyObject *args) {
     if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None) {
       PyErr_Clear();
       text = "None";
-    } else
+    }
+    else
       return NULL;
-  } else if (!ctx->pystring_to_string(o, text, true))
+  }
+  else if (!ctx->pystring_to_string(o, text, true))
     return NULL;
 
   grt::GRT::get()->send_output(text);
@@ -628,7 +635,7 @@ void PythonContext::printResult(std::map<std::string, std::string> &output) {
   }
 }
 
-#ifdef _WIN32
+#ifdef _MSC_VER
 static void printResultCallback(std::map<std::string, std::string> &output) {
   PythonContext *ctx;
 
@@ -650,7 +657,7 @@ static PyObject *getEventLogEntry(PyObject *self, PyObject *args) {
   if (!PyArg_ParseTuple(args, "ls", &seek, &query))
     return NULL;
 
-#ifdef _WIN32
+#ifdef _MSC_VER
   EventLogReader reader(query, printResultCallback);
   reader.SetPosition(seek);
   reader.ReadEvents();
@@ -770,10 +777,10 @@ static PyObject *grt_push_message_handler(PyObject *self, PyObject *args) {
   if (!PyCallable_Check(o))
     return NULL;
 
-  grt::GRT::get()->push_message_handler(
-    std::bind(&call_handle_message, std::placeholders::_1, std::placeholders::_2, AutoPyObject(o)));
+  grt::GRT::get()->pushMessageHandler(
+    new grt::SlotHolder(std::bind(&call_handle_message, std::placeholders::_1, std::placeholders::_2, AutoPyObject(o))));
 
-  return Py_BuildValue("i", grt::GRT::get()->message_handler_count());
+  return Py_BuildValue("i", grt::GRT::get()->messageHandlerCount());
 }
 
 static PyObject *grt_pop_message_handler(PyObject *self, PyObject *args) {
@@ -781,9 +788,9 @@ static PyObject *grt_pop_message_handler(PyObject *self, PyObject *args) {
   if (!(ctx = PythonContext::get_and_check()))
     return NULL;
 
-  grt::GRT::get()->pop_message_handler();
+  grt::GRT::get()->popMessageHandler();
 
-  return Py_BuildValue("i", grt::GRT::get()->message_handler_count());
+  return Py_BuildValue("i", grt::GRT::get()->messageHandlerCount());
 }
 
 //
@@ -1003,7 +1010,7 @@ void PythonContext::set_unwrap_pyobject_func(PyObject *(*func)(PyObject *, PyObj
  <li>etc
  */
 static PyMethodDef GrtModuleMethods[] = {
-  {"send_output", grt_send_output, METH_VARARGS, "Write a string in the GRT shell."},
+  {"send_output", grt_send_output, METH_VARARGS, "Write a string in the GRT shell." },
   {"write", grt_print, METH_VARARGS, "Write a string in the GRT shell (alias to send_output)."},
   {"send_error", grt_send_error, METH_VARARGS, "Write an error message to the GRT shell."},
   {"send_warning", grt_send_warning, METH_VARARGS, "Write a warning message to the GRT shell."},
@@ -1663,6 +1670,8 @@ static void create_class_wrapper(grt::MetaClass *meta, PyObject *locals) {
 int PythonContext::refresh() {
   WillEnterPython lock;
 
+  _grt_class_wrappers.clear();
+
   PyModule_AddObject(get_grt_module(), "root", from_grt(grt::GRT::get()->root()));
 
   PyObject *classes_dict = PyModule_GetDict(_grt_classes_module);
