@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,8 @@
 
 #include "grts/structs.db.mgmt.h"
 
+#include "wb_tunnel.h"
+
 #define DOC_DbMySQLQueryImpl                                                       \
   "Query execution and utility routines for  MySQL servers.\n"                     \
   "\n"                                                                             \
@@ -259,7 +261,7 @@ private:
   base::Mutex _mutex;
   std::map<int, ConnectionInfo::Ref> _connections;
   std::map<int, sql::ResultSet *> _resultsets;
-  std::map<int, std::shared_ptr<sql::TunnelConnection> > _tunnels;
+  std::map<int, std::shared_ptr<wb::SSHTunnel> > _tunnels;
   std::string _last_error;
   int _last_error_code;
 
@@ -359,7 +361,7 @@ int DbMySQLQueryImpl::execute(int conn, const std::string &query) {
   }
 
   try {
-    std::auto_ptr<sql::Statement> pstmt(con->createStatement());
+    std::unique_ptr<sql::Statement> pstmt(con->createStatement());
     int r = pstmt->execute(query) ? 1 : 0;
     cinfo->last_update_count = (size_t)pstmt->getUpdateCount();
     return r;
@@ -392,7 +394,7 @@ int DbMySQLQueryImpl::executeQuery(int conn, const std::string &query) {
   }
 
   try {
-    std::auto_ptr<sql::Statement> pstmt(con->createStatement());
+    std::unique_ptr<sql::Statement> pstmt(con->createStatement());
     pstmt->execute(query);
 
     sql::ResultSet *res;
@@ -442,7 +444,7 @@ grt::IntegerListRef DbMySQLQueryImpl::executeQueryMultiResult(int conn, const st
   grt::IntegerListRef result(grt::Initialized);
 
   try {
-    std::auto_ptr<sql::Statement> pstmt(con->createStatement());
+    std::unique_ptr<sql::Statement> pstmt(con->createStatement());
     pstmt->execute(query);
 
     try {
@@ -668,7 +670,7 @@ int DbMySQLQueryImpl::loadSchemata(int conn, grt::StringListRef schemata) {
 
   try {
     sql::DatabaseMetaData *dbc_meta(con->getMetaData());
-    std::auto_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", "", "schema"));
+    std::unique_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", "", "schema"));
     while (rset->next()) {
       std::string name = rset->getString("name");
       schemata.insert(name);
@@ -705,8 +707,8 @@ grt::DictRef DbMySQLQueryImpl::getServerVariables(int conn) {
   }
 
   try {
-    std::auto_ptr<sql::Statement> pstmt(con->createStatement());
-    std::auto_ptr<sql::ResultSet> rset(pstmt->executeQuery("show variables"));
+    std::unique_ptr<sql::Statement> pstmt(con->createStatement());
+    std::unique_ptr<sql::ResultSet> rset(pstmt->executeQuery("show variables"));
 
     while (rset->next()) {
       std::string name = rset->getString("Variable_name");
@@ -759,7 +761,7 @@ int DbMySQLQueryImpl::loadSchemaObjects(int conn, grt::StringRef schema, grt::St
     }
     sql::DatabaseMetaData *dbc_meta(con->getMetaData());
     for (std::list<std::string>::const_iterator i = object_types.begin(), end = object_types.end(); i != end; ++i) {
-      std::auto_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", *schema, *object_type));
+      std::unique_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", *schema, *object_type));
       while (rset->next()) {
         std::string name = rset->getString("name");
         std::string ddl = rset->getString("ddl");
@@ -790,7 +792,7 @@ grt::DictRef DbMySQLQueryImpl::loadSchemaObjectList(int conn, grt::StringRef sch
 
 int DbMySQLQueryImpl::openTunnel(const db_mgmt_ConnectionRef &info) {
   sql::DriverManager *dm = sql::DriverManager::getDriverManager();
-  std::shared_ptr<sql::TunnelConnection> tun = dm->getTunnel(info);
+  std::shared_ptr<wb::SSHTunnel> tun = dm->getTunnel(info);
   if (tun) {
     _tunnels[++_tunnel_id] = tun;
     return _tunnel_id;
@@ -801,7 +803,7 @@ int DbMySQLQueryImpl::openTunnel(const db_mgmt_ConnectionRef &info) {
 int DbMySQLQueryImpl::getTunnelPort(int tunnel) {
   if (_tunnels.find(tunnel) == _tunnels.end())
     throw std::invalid_argument("Invalid tunnel-id");
-  return _tunnels[tunnel]->get_port();
+  return _tunnels[tunnel]->getConfig().localport;
 }
 
 int DbMySQLQueryImpl::closeTunnel(int tunnel) {
